@@ -1,4 +1,4 @@
-/* $Id: ml_glib.c,v 1.15 2001/02/15 10:06:06 garrigue Exp $ */
+/* $Id: ml_glib.c,v 1.20 2002/08/07 03:56:36 garrigue Exp $ */
 
 #include <glib.h>
 #include <caml/mlvalues.h>
@@ -8,6 +8,9 @@
 
 #include "wrappers.h"
 #include "ml_glib.h"
+#include "glib_tags.h"
+
+#include "glib_tags.c"
 
 /* Utility functions */
 value copy_string_and_free (char *str)
@@ -44,8 +47,6 @@ value Val_GList (GList *list, value (*func)(gpointer))
     return cell;
 }
 
-value ml_get_null (value unit) { return 0L; }
-
 GList *GList_val (value list, gpointer (*func)(value))
 {
     CAMLparam1(list);
@@ -56,24 +57,7 @@ GList *GList_val (value list, gpointer (*func)(value))
     CAMLreturn (res);
 }
 
-/* Redirect printers */
-/* Currently broken for warning */
-static value ml_warning_handler = 0L;
-
-static void ml_warning_wrapper (const gchar *msg)
-{
-    value arg = copy_string ((char*)msg);
-    callback (ml_warning_handler, arg);
-}
-    
-value ml_g_set_warning_handler (value clos)
-{
-    value old_handler = ml_warning_handler ? ml_warning_handler : clos;
-    if (!ml_warning_handler) register_global_root (&ml_warning_handler);
-    g_set_warning_handler (ml_warning_wrapper);
-    ml_warning_handler = clos;
-    return old_handler;
-}
+/* Redirect printer */
 
 static value ml_print_handler = 0L;
 
@@ -83,7 +67,7 @@ static void ml_print_wrapper (const gchar *msg)
     callback (ml_print_handler, arg);
 }
     
-value ml_g_set_print_handler (value clos)
+CAMLprim value ml_g_set_print_handler (value clos)
 {
     value old_handler = ml_print_handler ? ml_print_handler : clos;
     if (!ml_print_handler) register_global_root (&ml_print_handler);
@@ -109,11 +93,50 @@ ML_1 (g_main_quit, GMainLoop_val, Unit)
 ML_1 (g_main_destroy, GMainLoop_val, Unit)
 
 
+/* GIOChannel */
 
+Make_Val_final_pointer (GIOChannel, g_io_channel_ref, g_io_channel_unref, 0)
+Make_Val_final_pointer_ext (GIOChannel, _noref, Ignore, g_io_channel_unref, 20)
+#define GIOChannel_val(val) ((GIOChannel*)Pointer_val(val))
+
+#ifndef _WIN32
+ML_1 (g_io_channel_unix_new, Int_val, Val_GIOChannel_noref)
+#else
+CAMLprim value ml_g_io_channel_unix_new(value v)
+{  invalid_argument("Glib.channel_unix_new: not implemented"); return 1; }
+#endif
+
+gboolean ml_g_io_channel_watch(GIOChannel *s, GIOCondition c, gpointer data)
+{
+    value *clos_p = (value*)data;
+    return Val_int(callback(*clos_p, Val_unit));
+}
+void ml_g_destroy_notify(gpointer data)
+{
+    ml_global_root_destroy(data);
+}
+
+CAMLprim value ml_g_io_add_watch(value cond, value clos, value prio, value io)
+{
+    g_io_add_watch_full(GIOChannel_val(io),
+                        Option_val(prio,Int_val,0),
+                        Io_condition_val(cond),
+                        ml_g_io_channel_watch,
+                        ml_global_root_new(clos),
+                        ml_g_destroy_notify);
+    return Val_unit;
+}
+
+/* Thread initialization ? */
+/*
+ML_1(g_thread_init, NULL Ignore, Unit)
+ML_0(gdk_threads_enter, Unit)
+ML_0(gdk_threads_leave, Unit)
+*/
 
 /* This is not used, but could be someday... */
 /*
-value Val_GSList (GSList *list, value (*func)(gpointer))
+CAMLprim value Val_GSList (GSList *list, value (*func)(gpointer))
 {
     value new_cell, result, last_cell, cell;
 
